/* ******************************************************************************

  CSERun.c	--	Batch scheduling Upload and Download utility program for CSE
  encyclopedia.

  Author	--	Sergey Gerasimov, sergey@jps.net
  License	--	Freeware.  

  Usage		--	cserun <download file name> <upload fle name>

******************************************************************************* */

/*

For maximum number of connections to your encyclopedia, consult the encyclopedia
configuration file (iefmd.ini in the cse folder).

  Here is our configuration:

	            ; Download Utility
		        [Program]
                AppName      = UTLDOWN
                NodeName     = IHSSCSE1
                RunFile      = "e:\coolcse\utldown"
                Parameters   = ""
                LogLocation  = "e:\coolcse\down"
                NbrLogFiles  = 1
                LimitEachLog = 5
                LogLevel     = STANDARD
                AutoStart    = ONDEMAND
                MinInstance  = 1
                MaxInstance  = 3          < ------- MAX number of connections
                QueueDepth   = 1


Please, do not specify file names longer than 8 bytes and containing spaces.
Sterling utilities do not like that.

Sterling's download tends to wait for the enter key press if something is wrong in the command line
passed to it.  I was not successful in DUPing the stderr from the father process.  If you know
how to do it in NT, please let me know.  That is the reson why you see the ENTER_KEY_FILE in the code.
Create this file and fill it with new line characters.

This program will automatically put quotes around your model and subset names to allow for spaces.

*/

#include <string.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <errno.h>
#include <stdlib.h>
#include <direct.h>
#include <time.h>

// Modify the following definitions to fit your environment

#define DFLT_CODE_PAGE	1252
#define	ENCY_DOWNLOAD	"e:\\coolcse\\download.exe"
#define	ENCY_UPLOAD		"e:\\coolcse\\upload.exe"
#define	BACKUP_FILE		"backup.run"
#define	ENTER_KEY_FILE	"< e:\\sergey\\enter.key"  // please keep the redefinition of the imput ('<')
#define ENCY_MAX_CONN	3	                       // MAX number of simultanious connections to your ency

// Decalarations

void parse	(char	*, char	[10][250]);				//	parse delimited line
void push	(int  [ENCY_MAX_CONN], int);			//	push PID on queue

// Main body of the program

void main(int argc, char *argv[])
{

FILE *filed, *fileup, *fb;
char str[10][250], buffer[250], comments[100][250], datetime[2][20];
int i, counter = 0, queue[ENCY_MAX_CONN], com = 0;

if (argc < 3) { printf("Usage: %s <download file name> <upload file name>\n",argv[0]); return; }

//	Open input files

if ( (filed = fopen( argv[1], "r+t")) == NULL ) { 
	printf("Unable to open download file: %s\n", argv[1]);
	return;
}

if ( (fileup = fopen( argv[2], "r+t")) == NULL ) {
	printf("Unable to open upload file: %s\n", argv[2]);
	fclose(filed);
	return;
}

if ( (fb = fopen(BACKUP_FILE, "a")) == NULL ) {
	printf("Unable to open file: %s\n", BACKUP_FILE);
	fclose(filed);
	fclose(fileup);
	return;
}

// Upload section

	sprintf(buffer,"\n[ Upload %s %s ]\n", _strdate(datetime[0]), _strtime(datetime[1]));
	fputs(buffer, fb);
	
	while ( fgets( buffer, 250, fileup) != NULL ) {
			if (buffer[0] == ';') { strcpy(comments[ (com < 100) ? com++ : (com = 100)], buffer); continue; }
			fputs(buffer, fb);
			for (i = 0; i < (int) strlen(buffer); i++) if (buffer[i] == '\n') { buffer[i] = '\0'; break; }
			strcpy(str[0], "-u");
			strcpy(str[2], "-d");
			parse(buffer, str);
			printf("running upload for %s %s\n", str[1], str[3]);
			spawnl(_P_WAIT, (const char *)ENCY_UPLOAD, str[0], str[1], str[2], str[3], NULL);
			}

// Complete upload process

		fclose(fileup);
		
		fileup = fopen( argv[2], "w+t");
		for (i = 0; i < com; i++) 
			fputs(comments[i], fileup);

		fclose(fileup);

// Download section

		com = counter = 0;
		for (i=0;i<ENCY_MAX_CONN;queue[i++]=0);

		sprintf(buffer,"\n[ Download %s %s ]\n", _strdate(datetime[0]), _strtime(datetime[1]));
		fputs(buffer, fb);

		while ( fgets( buffer, 250, filed) != NULL ) {
			if (buffer[0] == ';') { strcpy(comments[ (com < 100) ? com++ : (com = 100)], buffer); continue; }
			fputs(buffer, fb);
			for (i = 0; i < (int)strlen(buffer); i++) if (buffer[i] == '\n') { buffer[i] = '\0'; break; }
			strcpy(str[0], "-u");
			strcpy(str[2], "-m");
			strcpy(str[4], "-s");
			strcpy(str[6], "-d");
			strcpy(str[8], "-i");
			sprintf(str[9],"%d", DFLT_CODE_PAGE);
			
			parse(buffer, str);
			sprintf(buffer,"mkdir %s", str[7]);
			system(buffer);
			printf("running download for %s %s %s\n", str[1], str[3], str[5]);

			push( queue, (int)spawnl(_P_NOWAIT, (const char * )ENCY_DOWNLOAD,
				str[0], str[1], str[2], 
				str[3], str[4], str[5],
				str[6], str[7], str[8],
				str[9], ENTER_KEY_FILE, NULL));

			if (++counter >= ENCY_MAX_CONN) {
				printf("waiting for %d\n", queue[ENCY_MAX_CONN - 1]);
				_cwait((int *)NULL, queue[ENCY_MAX_CONN - 1], (int )NULL);
				}

			}

// Complete download process

		fclose(filed);
		
		filed = fopen( argv[1], "w+t");
		
		for (i = 0; i < com; i++) 
			fputs(comments[i], filed);

		fclose(filed);

		fclose(fb);
}

//	Utility functions

void parse(char *src, char dest[10][250])
{
	char buffer[250];
	int i,j=0,k=1;

	memset(buffer, '\0', 250);
	for (i = 0; i < (int) strlen(src) && k <= 9; i++)
		if(src[i] != ',') buffer[j++] = src[i];
			else {
				if (k == 3 || k == 5 || k == 7) sprintf(dest[k],"\"%s\"",buffer);
				else							strcpy(dest[k], buffer);
			k+=2; j=0;
			memset(buffer, '\0', 250);
			}
	strcpy(dest[k], buffer);
}

void push(int queue[ENCY_MAX_CONN], int handle)
{
int i;
for (i = ENCY_MAX_CONN; i >= 0; queue[i--] = (!i) ? handle : queue[i-1]);
}
